@@ -54,8 +54,6 @@
 #include "service.h"
 #include "transport.h"
 
-static GSList *sessions = NULL;
-
 typedef struct {
 	uint8_t  version;
 	uint8_t  flags;
@@ -84,58 +82,12 @@ static struct {
 	{ 0xFF,			NULL		},
 };
 
-/* Possible Response */
-static struct {
-	int rsp;
-	const char *name;
-} obex_response[] = {
-	{ G_OBEX_RSP_CONTINUE,			"CONTINUE"		},
-	{ G_OBEX_RSP_SUCCESS,			"SUCCESS"		},
-	{ G_OBEX_RSP_CREATED,			"CREATED"		},
-	{ G_OBEX_RSP_ACCEPTED,			"ACCEPTED"		},
-	{ G_OBEX_RSP_NON_AUTHORITATIVE,		"NON_AUTHORITATIVE"	},
-	{ G_OBEX_RSP_NO_CONTENT,		"NO_CONTENT"		},
-	{ G_OBEX_RSP_RESET_CONTENT,		"RESET_CONTENT"		},
-	{ G_OBEX_RSP_PARTIAL_CONTENT,		"PARTIAL_CONTENT"	},
-	{ G_OBEX_RSP_MULTIPLE_CHOICES,		"MULTIPLE_CHOICES"	},
-	{ G_OBEX_RSP_MOVED_PERMANENTLY,		"MOVED_PERMANENTLY"	},
-	{ G_OBEX_RSP_MOVED_TEMPORARILY,		"MOVED_TEMPORARILY"	},
-	{ G_OBEX_RSP_SEE_OTHER,			"SEE_OTHER"		},
-	{ G_OBEX_RSP_NOT_MODIFIED,		"NOT_MODIFIED"		},
-	{ G_OBEX_RSP_USE_PROXY,			"USE_PROXY"		},
-	{ G_OBEX_RSP_BAD_REQUEST,		"BAD_REQUEST"		},
-	{ G_OBEX_RSP_UNAUTHORIZED,		"UNAUTHORIZED"		},
-	{ G_OBEX_RSP_PAYMENT_REQUIRED,		"PAYMENT_REQUIRED"	},
-	{ G_OBEX_RSP_FORBIDDEN,			"FORBIDDEN"		},
-	{ G_OBEX_RSP_NOT_FOUND,			"NOT_FOUND"		},
-	{ G_OBEX_RSP_METHOD_NOT_ALLOWED,	"METHOD_NOT_ALLOWED"	},
-	{ G_OBEX_RSP_NOT_ACCEPTABLE,		"NOT_ACCEPTABLE"	},
-	{ G_OBEX_RSP_PROXY_AUTH_REQUIRED,	"PROXY_AUTH_REQUIRED"	},
-	{ G_OBEX_RSP_REQUEST_TIME_OUT,		"REQUEST_TIME_OUT"	},
-	{ G_OBEX_RSP_CONFLICT,			"CONFLICT"		},
-	{ G_OBEX_RSP_GONE,			"GONE"			},
-	{ G_OBEX_RSP_LENGTH_REQUIRED,		"LENGTH_REQUIRED"	},
-	{ G_OBEX_RSP_PRECONDITION_FAILED,	"PRECONDITION_FAILED"	},
-	{ G_OBEX_RSP_REQ_ENTITY_TOO_LARGE,	"REQ_ENTITY_TOO_LARGE"	},
-	{ G_OBEX_RSP_REQ_URL_TOO_LARGE,		"REQ_URL_TOO_LARGE"	},
-	{ G_OBEX_RSP_UNSUPPORTED_MEDIA_TYPE,	"UNSUPPORTED_MEDIA_TYPE"},
-	{ G_OBEX_RSP_INTERNAL_SERVER_ERROR,	"INTERNAL_SERVER_ERROR"	},
-	{ G_OBEX_RSP_NOT_IMPLEMENTED,		"NOT_IMPLEMENTED"	},
-	{ G_OBEX_RSP_BAD_GATEWAY,		"BAD_GATEWAY"		},
-	{ G_OBEX_RSP_SERVICE_UNAVAILABLE,	"SERVICE_UNAVAILABLE"	},
-	{ G_OBEX_RSP_GATEWAY_TIMEOUT,		"GATEWAY_TIMEOUT"	},
-	{ G_OBEX_RSP_VERSION_NOT_SUPPORTED,	"VERSION_NOT_SUPPORTED"	},
-	{ G_OBEX_RSP_DATABASE_FULL,		"DATABASE_FULL"		},
-	{ G_OBEX_RSP_DATABASE_LOCKED,		"DATABASE_LOCKED"	},
-	{ 0xFF,					NULL			},
-};
-
 static gboolean handle_async_io(void *object, int flags, int err,
 						void *user_data);
 
-static void print_event(int cmd, int rsp)
+static void print_event(int cmd, uint8_t rsp)
 {
-	const char *cmdstr = NULL, *rspstr = NULL;
+	const char *cmdstr = NULL;
 	int i;
 	static int lastcmd;
 
@@ -150,13 +102,8 @@ static void print_event(int cmd, int rsp)
 		cmdstr = obex_command[i].name;
 	}
 
-	for (i = 0; obex_response[i].rsp != 0xFF; i++) {
-		if (obex_response[i].rsp != rsp)
-			continue;
-		rspstr = obex_response[i].name;
-	}
-
-	obex_debug("%s(0x%x), %s(0x%x)", cmdstr, cmd, rspstr, rsp);
+	obex_debug("%s(0x%x), %s(0x%x)", cmdstr, cmd,
+						g_obex_strerror(rsp), rsp);
 }
 
 static void os_set_response(struct obex_session *os, int err)
@@ -233,8 +180,6 @@ static void os_reset_session(struct obex_session *os)
 
 static void obex_session_free(struct obex_session *os)
 {
-	sessions = g_slist_remove(sessions, os);
-
 	if (os->io) {
 		g_io_channel_shutdown(os->io, TRUE, NULL);
 		g_io_channel_unref(os->io);
@@ -783,6 +728,9 @@ int obex_put_stream_start(struct obex_session *os, const char *filename)
 		return err;
 	}
 
+	if (os->size != OBJECT_SIZE_DELETE && os->size != OBJECT_SIZE_UNKNOWN)
+		manager_emit_transfer_property(os->service_data, "Size");
+
 	os->path = g_strdup(filename);
 
 	return 0;
@@ -1067,8 +1015,6 @@ int obex_session_start(GIOChannel *io, uint16_t tx_mtu, uint16_t rx_mtu,
 	obex_getsockname(os, &os->src);
 	obex_getpeername(os, &os->dst);
 
-	sessions = g_slist_prepend(sessions, os);
-
 	return 0;
 }
 
